/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package cin.aquario;

import org.simulare.*;

/**
 * Defines the breathing action. Coral and fishes breath the oxigen from the
 * aquarium.
 */
public class ActionBreath extends AbstractTask {
    public static final String PRODUCER = "oxigen_level";
    public static final String CONSUMER = "oxigen_rate";

    public ActionBreath() {
        this("breath");
    }

    public ActionBreath(String name) {
        super(name);
    }

    public void execute(Item item) {
        Attribute att = item.getAttribute(LivingThing.IS_ALIVE);
        if( att != null && att.getBoolean() ) {
            System.out.println(item.getName() + ":Start breath");
            CompositeItem parent = item.getParent();
            if (parent != null) {
                Attribute level = parent.getAttribute(PRODUCER);
                Attribute rate = item.getAttribute(CONSUMER);
                if (level != null && rate != null) {
                    level.setValue(new Integer(level.getInt() - rate.getInt()));
                }
            }
            fireTask(item);
            System.out.println(item.getName() + ":End breath");
        }
    }
}