/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package cin.aquario;

import org.simulare.*;


public class ActionEat extends AbstractTask {

    private String food;
    private double radius;

    public ActionEat() {
        this("eat");
    }

    public ActionEat(String name) {
        super(name);
    }

    public String getFood() {
        return food;
    }

    public void setFood(String food) {
        this.food = food;
    }

    public double getRadius() {
        return radius;
    }

    public void setRadius(double radius) {
        this.radius = radius;
    }

    public void execute(Item item) {
        Attribute att = item.getAttribute(LivingThing.IS_ALIVE);
        if( att != null && att.getBoolean() ) {
            System.out.println(item.getName() + ":Start Eating");
            Attribute x = item.getAttribute(ActionMove.CONSUMERX);
            Attribute y = item.getAttribute(ActionMove.CONSUMERY);
            Attribute z = item.getAttribute(ActionMove.CONSUMERZ);

            Attribute tx, ty, tz;
            CompositeItem parent = item.getParent();
            if (parent != null) {
                for (int i = 0; i < parent.getItemsLength(); i++) {
                    Item child = parent.getItem(i);
                    if (child.getName().startsWith(food)) {
                        tx = child.getAttribute(ActionMove.CONSUMERX);
                        ty = child.getAttribute(ActionMove.CONSUMERY);
                        tz = child.getAttribute(ActionMove.CONSUMERZ);
                        double dist = Math.sqrt(
                                Math.pow(x.getDouble() - tx.getDouble(), 2) +
                                Math.pow(y.getDouble() - ty.getDouble(), 2) +
                                Math.pow(z.getDouble() - tz.getDouble(), 2)
                                );
                        System.out.println("EAT (" + radius + "):" + dist);
                        if (dist < radius) {
                            if (System.currentTimeMillis() % 2 == 0) {
                                Attribute attc = child.getAttribute(LivingThing.IS_ALIVE);
                                attc.setValue(Boolean.FALSE);
                                SimulationAquario.sim.removeItem(child);
                            }
                        }
                    }
                }
            }
            System.out.println(item.getName() + ":end Eating");
        }
    }
}