/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package cin.aquario;

import java.util.*;

import org.simulare.*;

/**
 * Sets up an attribute with a value in the range of min up to max.
 * By now, this classe only set up attributes of tipe "int" or "double".
 */
public class SetupRandom extends AbstractTask {

    private Random r;
    private String attribute;
    private double min;
    private double max;

    public SetupRandom() {
        this(null, Double.MIN_VALUE, Double.MAX_VALUE);
    }

    public SetupRandom(String attribute, double min, double max) {
        super("random");
        r = new Random();
        setAttribute(attribute);
        setMin(min);
        setMax(max);
    }

    public String getAttribute() {
        return attribute;
    }

    public void setAttribute(String attribute) {
        this.attribute = attribute;
    }

    public double getMin() {
        return min;
    }

    public void setMin(double min) {
        this.min = min;
    }

    public double getMax() {
        return max;
    }

    public void setMax(double max) {
        this.max = max;
    }

    public void execute(Item item) {
        Attribute att = item.getAttribute(attribute);
        if (att != null) {
            if (att.getType() == Integer.class) {
                att.setValue(new Integer((int) min +
                                         Math.abs(r.nextInt() %
                                                  (int) (max - min))));
            } else if (att.getType() == Double.class) {
                double n = r.nextDouble();
                Double d = new Double(min + (n / (1 / (max - min))));
                att.setValue(d);
            }
            System.out.println(item + "." + att + " set to " + att.getValue());
        }
    }
}