/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package cin.aquario;

import org.simulare.*;

public class SimulationAquario {

    public static Simulation sim;
    public static VisualTask view = new Report3D();

    static {
        sim = new Simulation();
        sim.addItem(Example.aquario);
        sim.addItem(Example.coral);
        sim.addItem(Example.presa);
        sim.addItem(Example.predador);

        Task breath = new ActionBreath();
        Task move = new ActionMove();
        Task print = new ReportPrintln();

        sim.addTask(breath);
        sim.addTask(move);
        sim.addTask(print);
        sim.addTask(view);

        // listener to finalize simulation when there is no more oxigen
        ItemListener listener = new NoOxigen();
        Example.aquario.addItemListener(listener);

//---------------- SET ALIVE  -----------------------------//
        Task addItem = new AddItem(Example.aquario);
        sim.addSchedule(new ScheduledTask(addItem, Example.coral, 0));
        sim.addSchedule(new ScheduledTask(addItem, Example.presa, 0));
        sim.addSchedule(new ScheduledTask(addItem, Example.predador, 0));
//---------------- SET ALIVE  -----------------------------//

//---------------- SET ALIVE  -----------------------------//
        Task setAlive = new SetupBoolean(LivingThing.IS_ALIVE, true);
        sim.addSchedule(new ScheduledTask(setAlive, Example.coral, 0));
        sim.addSchedule(new ScheduledTask(setAlive, Example.presa, 0));
        sim.addSchedule(new ScheduledTask(setAlive, Example.predador, 0));
//---------------- SET ALIVE  -----------------------------//

//---------------- BREATH -----------------------------//
        // setup aquario oxigen level
        Task setupAir = new SetupRandom(ActionBreath.PRODUCER, 1000, 1500);
        sim.addSchedule(new ScheduledTask(setupAir, Example.aquario, 0));
        // setup coral, fish and predator air consume rate
        Task consumeAirRate = new SetupRandom(ActionBreath.CONSUMER, 0, 10);
        sim.addSchedule(new ScheduledTask(consumeAirRate, Example.coral, 0));
        sim.addSchedule(new ScheduledTask(consumeAirRate, Example.presa, 0));
        sim.addSchedule(new ScheduledTask(consumeAirRate, Example.predador, 0));
        // schedule breath actions
        sim.addSchedule(new PeriodicTask(breath, Example.coral, 1, 1));
        sim.addSchedule(new PeriodicTask(breath, Example.presa, 2, 2));
        sim.addSchedule(new PeriodicTask(breath, Example.predador, 3, 3));
        // reports on item states each 1 cicle, start at cicle 0 and execute
        // to infinite.
        sim.addSchedule(new PeriodicTask(print, Example.aquario, 1, 1));
//---------------- BREATH -----------------------------//

//---------------- MOVE -----------------------------//
        double altura = Example.aquario.getAttribute("altura").getDouble();
        double largura = Example.aquario.getAttribute("largura").getDouble();
        double comprimento = Example.aquario.getAttribute("comprimento").getDouble();

        // setup X position based on the aquarium box
        Task setupX = new SetupRandom(ActionMove.CONSUMERX, 0, altura);
        sim.addSchedule(new ScheduledTask(setupX, Example.coral, 0));
        sim.addSchedule(new ScheduledTask(setupX, Example.presa, 0));
        sim.addSchedule(new ScheduledTask(setupX, Example.predador, 0));
        // setup Y position based on the aquarium box
        Task setupY = new SetupRandom(ActionMove.CONSUMERY, 0, largura);
        //sim.addSchedule(new ScheduledTask(setupY, Example.coral, 0));
        sim.addSchedule(new ScheduledTask(setupY, Example.presa, 0));
        sim.addSchedule(new ScheduledTask(setupY, Example.predador, 0));
        // setup Z position based on the aquarium box
        Task setupZ = new SetupRandom(ActionMove.CONSUMERZ, 0, comprimento);
        sim.addSchedule(new ScheduledTask(setupZ, Example.coral, 0));
        sim.addSchedule(new ScheduledTask(setupZ, Example.presa, 0));
        sim.addSchedule(new ScheduledTask(setupZ, Example.predador, 0));
        // schedule move actions
        sim.addSchedule(new PeriodicTask(move, Example.coral, 1, 3));
        sim.addSchedule(new PeriodicTask(move, Example.presa, 1, 2));
        sim.addSchedule(new PeriodicTask(move, Example.predador, 1, 1));
        // reports on item states each 1 cicle, start at cicle 1 and execute
        // to infinite.
        sim.addSchedule(new PeriodicTask(view, Example.aquario, 1, 1));
        sim.addSchedule(new PeriodicTask(view, Example.coral, 1, 1));
        sim.addSchedule(new PeriodicTask(view, Example.presa, 1, 1));
        sim.addSchedule(new PeriodicTask(view, Example.predador, 1, 1));
//---------------- MOVE -----------------------------//

//---------------- EAT -----------------------------//
        ActionEat eat = new ActionEat("fish_eats_coral");
        eat.setFood(Example.coral.getName());
        eat.setRadius(1);
        sim.addSchedule(new PeriodicTask(eat,Example.presa,1,1));

        eat = new ActionEat("predator_eats_coral");
        eat.setFood(Example.coral.getName());
        eat.setRadius(1.5);
        sim.addSchedule(new PeriodicTask(eat,Example.predador,1,1));

        eat = new ActionEat("predator_eats_fish");
        eat.setFood(Example.presa.getName());
        eat.setRadius(2);
        sim.addSchedule(new PeriodicTask(eat,Example.predador,1,1));
//---------------- EAT -----------------------------//
    }
}