/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

/**
 * Defines a default facility task to be extended by non-visual tasks.
 */
public abstract class AbstractTask extends Named implements Task {
    protected TaskEvent tevent = new TaskEvent(this);
    private transient Vector taskListeners;

    public AbstractTask() {
        this(null);
    }

    public AbstractTask(String name) {
        super(name);
    }

    public synchronized void removeTaskListener(TaskListener l) {
        if (taskListeners != null && taskListeners.contains(l)) {
            Vector v = (Vector) taskListeners.clone();
            v.removeElement(l);
            taskListeners = v;
        }
    }

    public synchronized void addTaskListener(TaskListener l) {
        Vector v = taskListeners == null ? new Vector(2) :
                   (Vector) taskListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            taskListeners = v;
        }
    }

    protected void fireTask(Item item) {
        tevent.item = item;
        fireTaskExecuted(tevent);
    }

    protected void fireTaskExecuted(TaskEvent e) {
        if (taskListeners != null) {
            Vector listeners = taskListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((TaskListener) listeners.elementAt(i)).taskExecuted(e);
            }
        }
    }
}