/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

/**
 * Represents an attribute of a simulation item.
 */
public class Attribute extends Named {
    private Class type;
    private Object value;
    private transient Vector attributeListeners;

    public Attribute(String name, Class type, Object value) {
        super(name);
        setType(type);
        setValue(value);
    }

    public Class getType() {
        return type;
    }

    public void setType(Class type) {
        AttributeEvent e = new AttributeEvent(this);
        e.oldClass = this.type;
        this.type = type;
        fireTypeChanged(e);
    }

    public Object getValue() {
        return value;
    }

    public void setValue(Object value) {
        AttributeEvent e = new AttributeEvent(this);
        e.oldValue = this.value;
        this.value = value;
        fireValueChanged(e);
    }

    public boolean getBoolean() {
        return ((Boolean) value).booleanValue();
    }

    public char getChar() {
        return ((Character) value).charValue();
    }

    public byte getByte() {
        return ((Number) value).byteValue();
    }

    public short getShort() {
        return ((Number) value).shortValue();
    }

    public int getInt() {
        return ((Number) value).intValue();
    }

    public long getLong() {
        return ((Number) value).longValue();
    }

    public float getFloat() {
        return ((Number) value).floatValue();
    }

    public double getDouble() {
        return ((Number) value).doubleValue();
    }

    public String getString() {
        return (String) value;
    }

    public Attribute getCopy() {
        return new Attribute(getName(), getType(), getValue());
    }

    public synchronized void removeAttributeListener(AttributeListener l) {
        if (attributeListeners != null && attributeListeners.contains(l)) {
            Vector v = (Vector) attributeListeners.clone();
            v.removeElement(l);
            attributeListeners = v;
        }
    }

    public synchronized void addAttributeListener(AttributeListener l) {
        Vector v = attributeListeners == null ? new Vector(2) :
                   (Vector) attributeListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            attributeListeners = v;
        }
    }

    protected void fireTypeChanged(AttributeEvent e) {
        if (attributeListeners != null) {
            Vector listeners = attributeListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((AttributeListener) listeners.elementAt(i)).typeChanged(e);
            }
        }
    }

    protected void fireValueChanged(AttributeEvent e) {
        if (attributeListeners != null) {
            Vector listeners = attributeListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((AttributeListener) listeners.elementAt(i)).valueChanged(e);
            }
        }
    }
}