/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

/**
 * Identify a clock based task. The task is executed over the object depending
 * on clock events.
 */
public abstract class ClockedTask implements ClockListener {
    private Task task;
    private Item item;

    public ClockedTask() {
        this(null,null);
    }

    public ClockedTask(Task task, Item item) {
        setTask(task);
        setItem(item);
    }

    public Task getTask() {
        return task;
    }

    public void setTask(Task task) {
        this.task = task;
    }

    public Item getItem() {
        return item;
    }

    public void setItem(Item item) {
        this.item = item;
    }

    public void clockStarted(ClockEvent e) {}

    public void clockPaused(ClockEvent e) {}

    public void clockResumed(ClockEvent e) {}

    public void clockStoped(ClockEvent e) {}

    public void clockChanged(ClockEvent e) {}
}
