/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

/**
 * An item which is a collection.
 */
public class CollectionItem extends Item {
    private List items_order;
    private Map items;

    public CollectionItem(String name) {
        super(name);
        items_order = new LinkedList();
        items = new HashMap();
    }

    public void addItem(Item item) {
        addItem(items_order.size(), item);
    }

    public void addItem(int index, Item item) {
        String name = item.getName();
        if (!items_order.contains(name)) {
            items_order.add(index, name);
        }
        items.put(name, item);
        fireItemStateChanged(event);
    }

    public Item removeItem(String name) {
        items_order.remove(name);
        Item result = (Item) items.remove(name);
        fireItemStateChanged(event);
        return result;
    }

    public Item removeItem(int index) {
        return removeItem((String) items_order.get(index));
    }

    public Item getItem(String name) {
        return (Item) items.get(name);
    }

    public Item getItem(int index) {
        return getItem((String) items_order.get(index));
    }

    public int getItemIndex(String name) {
        return items_order.indexOf(name);
    }

    public String getItemName(int index) {
        return (String) items_order.get(index);
    }

    public Iterator getItemsNames() {
        return items_order.iterator();
    }

    public int getItemsLength() {
        return items_order.size();
    }

    public boolean contains(Item item) {
        return getItem(item.getName()) != null;
    }
}
