/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

/**
 * Super-class for all object that receive a name.
 */
public abstract class Named implements Name {
    private String name;
    private NameEvent nevent = new NameEvent(this);
    private transient Vector nameListeners;

    public Named(String name) {
        setName(name);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String toString() {
        return getName();
    }

    public synchronized void removeNameListener(NameListener l) {
        if (nameListeners != null && nameListeners.contains(l)) {
            Vector v = (Vector) nameListeners.clone();
            v.removeElement(l);
            nameListeners = v;
        }
    }

    public synchronized void addNameListener(NameListener l) {
        Vector v = nameListeners == null ? new Vector(2) :
                   (Vector) nameListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            nameListeners = v;
        }
    }

    protected void fireName(Name name) {
        nevent.oldName = name.getName();
        fireNameChanged(nevent);
    }

    protected void fireNameChanged(NameEvent e) {
        if (nameListeners != null) {
            Vector listeners = nameListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((NameListener) listeners.elementAt(i)).nameChanged(e);
            }
        }
    }
}