/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

/**
 * Represents a Task scheduled to execute in periods, where the start and end
 * time can be specified.
 */
public class PeriodicTask extends ScheduledTask {
    private long period;
    private long end;

    public PeriodicTask(long start, long period) {
        this(null, null, start, period);
    }

    public PeriodicTask(long start, long period, long end) {
        this(null, null, start, period, end);
    }

    public PeriodicTask(Task task, Item item, long start, long period) {
        this(task, item, start, period, Long.MAX_VALUE);
    }

    public PeriodicTask(Task task, Item item, long start, long period, long end) {
        super(task, item, start);
        setPeriod(period);
        setEnd(end);
    }

    public long getPeriod() {
        return period;
    }

    public void setPeriod(long period) {
        this.period = period;
    }

    public long getEnd() {
        return end;
    }

    public void setEnd(long end) {
        this.end = end;
    }

    public void clockChanged(ClockEvent e) {
        // the clock can not have been stoped, if it does, the even must be
        // ignored.
        super.clockChanged(e);
        if (!e.stoped) {
            // if the timer has started the task, the event can be reexecuted
            // only when e.counter is greater than counter
            if (isDone() && e.counter > getStart()) {
                if (e.counter <= end) {
                    if (((e.counter - getStart()) % period) == 0) {
                        getTask().execute(getItem());
                    }
                }
            }
        }
    }
}