/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

/**
 * Represents a Task scheduled to execute on a specific time.
 */
public class ScheduledTask extends ClockedTask {
    private long start;
    private boolean done;

    public ScheduledTask(long start) {
        this(null,null,start);
    }

    public ScheduledTask(Task task, Item item, long start) {
        super(task, item);
        setStart(start);
        done = false;
    }

    public long getStart() {
        return start;
    }

    public void setStart(long start) {
        this.start = start;
    }

    public boolean isDone() {
        return done;
    }

    public void clockStarted(ClockEvent e) {
        done = false;
    }

    public void clockStoped(ClockEvent e) {
        done = false;
    }

    public void clockChanged(ClockEvent e) {
        // the clock can not have been stoped if it does, the even must be
        // ignored.
        if (!e.stoped) {
            if (!done) { // and this task was not already executed
                if (e.counter >= start) {
                    getTask().execute(getItem());
                    done = true;
                }
            }
        } else {
            /*
            System.out.println("Task(" + getTask().getClass().getName() +
                               ") on " + getItem().getName() +
                               " ignored. Clock was already stoped.");
            */
        }
    }
}