/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare;

import java.util.*;

public class Simulation {
    // simulation controler
    private Clock clock;

    // simulation items
    private Map items;

    // simulations tasks
    private Map tasks;

    // simulation actions
    private List schedule;

    public Simulation() {
        clock = new Clock();
        items = new HashMap();
        tasks = new HashMap();
        schedule = new LinkedList();
    }

    public Clock getClock() {
        return clock;
    }

    public void setClock(Clock clock) {
        this.clock = clock;
    }

    public void addItem(Item item) {
        items.put(item.getName(), item);
    }

    public void removeItem(Item item) {
        remove(null, item);
        items.remove(item.getName());
        CompositeItem parent = item.getParent();
        if( parent != null ) {
            parent.removeItem(item.getName());
            System.out.println(item+" removed from "+parent);
        }
    }

    private void remove(Task task, Item item) {
        List list = new LinkedList();
        Iterator iter = schedule.iterator();
        while (iter.hasNext()) {
            ClockedTask ct = (ClockedTask) iter.next();
            if (task != null && ct.getTask() == task) {
                list.add(ct);
            }
            if (item != null && ct.getItem() == item) {
                list.add(ct);
            }
        }
        iter = list.iterator();
        while (iter.hasNext()) {
            removeSchedule((ClockedTask) iter.next());
        }
    }

    public void addTask(Task task) {
        tasks.put(task.getName(), task);
    }

    public void removeTask(Task task) {
        remove(task, null);
        tasks.remove(task.getName());
    }

    public void addSchedule(ClockedTask ct) {
        if (clock != null) {
            clock.addClockListener(ct);
            schedule.add(ct);
        }
    }

    public void removeSchedule(ClockedTask ct) {
        System.out.println("Remove:" + ct.getTask() + " on " + ct.getItem());
        schedule.remove(ct);
    }
}