/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import org.simulare.*;

/**
 * A graphial representation of a clock.
 */
public class ClockPanel extends JPanel implements ClockListener {
    JLabel jLabelClock = new JLabel();
    JSpinner jSpinnerClock = new JSpinner();
    JLabel jLabelText = new JLabel();
    JButton jButtonStop = new JButton();
    JButton jButtonPlay = new JButton();
    JButton jButtonPause = new JButton();
    JPanel jPanelControl = new JPanel();
    ImageIcon play16;
    ImageIcon pause16;
    ImageIcon stop16;
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    GridBagLayout gridBagLayout2 = new GridBagLayout();
    JLabel jLabelStatus = new JLabel();
    JLabel jLabelCounter = new JLabel();
    JLabel jLabelValue = new JLabel();
    private Clock clock;
    JLabel jLabelMaxCounter = new JLabel();
    JTextField jTextFieldMaxCounter = new JTextField();

    public ClockPanel() {
        try {
            jbInit();
            SpinnerNumberModel model = new SpinnerNumberModel();
            model.setMinimum(Integer.valueOf("1"));
            model.setStepSize(new Integer(100));
            jSpinnerClock.setModel(model);
            setClock(new Clock());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        play16 = new ImageIcon(org.simulare.gui.ClockPanel.class.getResource(
                "Play16.gif"));
        pause16 = new ImageIcon(org.simulare.gui.ClockPanel.class.getResource(
                "Pause16.gif"));
        stop16 = new ImageIcon(org.simulare.gui.ClockPanel.class.getResource(
                "Stop16.gif"));
        jLabelClock.setText("Tick each:");
        this.setLayout(gridBagLayout1);
        jLabelText.setText("mls");
        jSpinnerClock.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                jSpinnerClock_stateChanged(e);
            }
        });
        jButtonStop.setEnabled(false);
        jButtonStop.setToolTipText("Stops the clock counting.");
        jButtonStop.setIcon(stop16);
        jButtonStop.setText("");
        jButtonStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonStop_actionPerformed(e);
            }
        });
        jButtonPlay.setToolTipText("Starts the clock counting.");
        jButtonPlay.setIcon(play16);
        jButtonPlay.setText("");
        jButtonPlay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonPlay_actionPerformed(e);
            }
        });
        jButtonPause.setEnabled(false);
        jButtonPause.setToolTipText("Pauses the clock.");
        jButtonPause.setIcon(pause16);
        jButtonPause.setText("");
        jButtonPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonPause_actionPerformed(e);
            }
        });
        jPanelControl.setLayout(gridBagLayout2);
        jPanelControl.setMinimumSize(new Dimension(0, 0));
        jLabelStatus.setToolTipText("");
        jLabelStatus.setText("<Here goes clock status>");
        this.setBorder(new TitledBorder(BorderFactory.createEtchedBorder(Color.
                white, new Color(148, 145, 140)), "Clock manager:"));
        jLabelCounter.setRequestFocusEnabled(true);
        jLabelCounter.setText("Counter:");
        jLabelValue.setForeground(new Color(91, 74, 157));
        jLabelValue.setText("0");
        jLabelMaxCounter.setText("N. of ticks:");
        jTextFieldMaxCounter.setText("");
        jTextFieldMaxCounter.setColumns(4);
        jTextFieldMaxCounter.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(FocusEvent e) {
                jTextFieldMaxCounter_focusLost(e);
            }
        });
        jTextFieldMaxCounter.addActionListener(new java.awt.event.
                                               ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jTextFieldMaxCounter_actionPerformed(e);
            }
        });
        jSpinnerClock.setToolTipText("Sets the tick real time interval.");
        this.add(jLabelClock, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        this.add(jSpinnerClock, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 2, 0, 5), 30, 0));
        this.add(jLabelText, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 5), 0, 0));
        this.add(jPanelControl, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 1));
        jPanelControl.add(jButtonPlay,
                          new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0
                                                 , GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(0, 0, 0, 0), 0, 1));
        jPanelControl.add(jButtonPause,
                          new GridBagConstraints(1, 0, 1, 1, 1.0, 1.0
                                                 , GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(0, 0, 0, 0), 0, 1));
        jPanelControl.add(jButtonStop,
                          new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                                                 , GridBagConstraints.CENTER,
                                                 GridBagConstraints.NONE,
                                                 new Insets(0, 0, 0, 0), 0, 1));
        this.add(jLabelStatus, new GridBagConstraints(2, 1, 2, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 5), 0, 0));
        this.add(jLabelCounter, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        this.add(jLabelValue, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 5, 0, 0), 0, 0));
        this.add(jLabelMaxCounter, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0
                , GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 0), 0, 0));
        this.add(jTextFieldMaxCounter,
                 new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
                                        , GridBagConstraints.WEST,
                                        GridBagConstraints.NONE,
                                        new Insets(0, 2, 0, 0), 0, 0));
    }

    /**
     * Return the associated clock .
     * @return The current clock.
     */
    public Clock getClock() {
        return clock;
    }

    /**
     * Sets a clock to adjust visually.
     * @param clock The clock to be setted.
     */
    public void setClock(Clock clock) {
        if (this.clock != null) {
            this.clock.removeClockListener(this);
        }
        this.clock = clock;
        this.clock.addClockListener(this);
        jSpinnerClock.setValue(new Long(clock.getPeriod()));
    }

    void jSpinnerClock_stateChanged(ChangeEvent e) {
        Number n = (Number) jSpinnerClock.getValue();
        clock.setPeriod(n.longValue());
        jLabelStatus.setText("Tick set to " + n + " mls.");
    }

    void jTextFieldMaxCounter_actionPerformed(ActionEvent e) {
        setMax();
    }

    void jTextFieldMaxCounter_focusLost(FocusEvent e) {
        setMax();
    }

    private void setMax() {
        String str = jTextFieldMaxCounter.getText();
        if ("".equals(str)) {
            clock.setMaxCounter(Long.MAX_VALUE);
            jLabelStatus.setText("N. of ticks set to +infinite.");
        } else {
            try {
                long max = Long.parseLong(str);
                clock.setMaxCounter(max);
                jLabelStatus.setText("N. of ticks set to " + max + ".");
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(this, "The number of ticks must be a valid number.\nIf the field is empty the number of tick will be set to the positive infinite value.",
                                              "Number of ticks",
                                              JOptionPane.INFORMATION_MESSAGE);
                if (clock.getMaxCounter() != Long.MAX_VALUE) {
                    jTextFieldMaxCounter.setText(String.valueOf(clock.
                            getMaxCounter()));
                }
            }
        }
    }

    void jButtonPlay_actionPerformed(ActionEvent e) {
        if (clock.isPaused()) {
            clockResumed(null);
            System.out.println("RESUME:" + System.currentTimeMillis());
            clock.resume();
        } else {
            clockStarted(null);
            System.out.println("START:" + System.currentTimeMillis());
            clock.start();
        }
    }

    void jButtonPause_actionPerformed(ActionEvent e) {
        clockPaused(null);
        System.out.println("PAUSE:" + System.currentTimeMillis());
        clock.pause();
    }

    void jButtonStop_actionPerformed(ActionEvent e) {
        clockStoped(null);
        System.out.println("STOP:" + System.currentTimeMillis());
        clock.stop();
    }

    public void clockStarted(ClockEvent e) {
        jSpinnerClock.setEnabled(false);
        jButtonPlay.setEnabled(false);
        jButtonPause.setEnabled(true);
        jButtonStop.setEnabled(true);
        jLabelStatus.setText("Clock started.");
    }

    public void clockPaused(ClockEvent e) {
        jSpinnerClock.setEnabled(true);
        jButtonPlay.setEnabled(true);
        jButtonPause.setEnabled(false);
        jLabelStatus.setText("Clock paused.");
    }

    public void clockResumed(ClockEvent e) {
        //on gui it has the same behavior
        clockStarted(e);
        // only the text changes
        jLabelStatus.setText("Clock resumed.");
    }

    public void clockStoped(ClockEvent e) {
        jSpinnerClock.setEnabled(true);
        jButtonPlay.setEnabled(true);
        jButtonPause.setEnabled(false);
        jButtonStop.setEnabled(false);
        jLabelStatus.setText("Clock finalized.");
    }

    public void clockChanged(ClockEvent e) {
        jLabelValue.setText(String.valueOf(e.counter));
    }

}