/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare.gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.tree.*;

import org.simulare.*;

/**
 * The composite item GUI representation.
 */
public class CompositeItemPanel extends JSplitPane {
    JPanel jPanel1 = new JPanel();
    JPanel jPanel2 = new JPanel();
    JScrollPane jScrollPane1 = new JScrollPane();
    CompositeItem rootItem = cin.aquario.Example.aquario; //new CompositeItem("environment");
    CompositeItemTreeModel model = new CompositeItemTreeModel(rootItem);
    JTreeSmart jTreeComposite = new JTreeSmart(model);
    BorderLayout borderLayout1 = new BorderLayout();
    ItemPanel jPanelItem = new ItemPanel(rootItem);
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    CompositeItemPopup popup = new CompositeItemPopup();
    JTreeSmartEvent last = null;
    int count = 0;
    public CompositeItemPanel() {
        try {
            jbInit();
            jTreeComposite.putClientProperty("JTree.lineStyle", "Angled");

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        jPanel1.setLayout(borderLayout1);
        jPanel2.setLayout(gridBagLayout1);
        popup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                popup_actionPerformed(e);
            }
        });
        jTreeComposite.addJTreeSmartListener(new org.simulare.gui.
                                             JTreeSmartListener() {
            public void singleClick(JTreeSmartEvent e) {
                jTreeComposite_singleClick(e);
            }

            public void doubleClick(JTreeSmartEvent e) {
            }

            public void popupClick(JTreeSmartEvent e) {
                jTreeComposite_popupClick(e);
            }
        });
        this.setOneTouchExpandable(true);
        this.add(jPanel1, JSplitPane.LEFT);
        jPanel1.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(jTreeComposite, null);
        this.add(jPanel2, JSplitPane.RIGHT);
        jPanel2.add(jPanelItem, new GridBagConstraints(0, 0, 1, 1, 0.8, 0.8
                , GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                new Insets(20, 0, 0, 0), 0, 0));
        this.setDividerLocation(150);
    }

    void jTreeComposite_popupClick(JTreeSmartEvent e) {
        last = e;
        popup.show(jTreeComposite, e.event.getX(), e.event.getY());
    }

    void popup_actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
        if ("Expand Current".equals(action)) {
            jTreeComposite.expandCurrent();
        } else if ("Expand All".equals(action)) {
            jTreeComposite.expandAll();
        } else if ("Colapse Current".equals(action)) {
            jTreeComposite.colapseCurrent();
        } else if ("Colapse All".equals(action)) {
            jTreeComposite.colapseAll();
        } else if ("Add Item".equals(action)) {
            Item node = (Item) last.path.getLastPathComponent();
            Item novo = new Item("item." + count++);
            if (!(node instanceof CompositeItem)) {
                TreePath parent = last.path.getParentPath();
                CompositeItem p = (CompositeItem) parent.getLastPathComponent();
                int index = p.getItemIndex(node.getName());
                p.removeItem(node.getName());
                CompositeItem f = node.toComposite();
                //the path now includes the new compositeItem.
                last.path = parent.pathByAddingChild(f);
                p.addItem(index, f);
                f.addItem(novo);
            } else {
                ((CompositeItem) node).addItem(novo);
            }
            jTreeComposite.setModel(model);
            TreePath t = last.path.pathByAddingChild(novo);
            jTreeComposite.updateUI();
            jTreeComposite.expandAll();
            jTreeComposite.setSelectionPath(t);
        } else if ("Remove Item".equals(action)) {
            TreePath[] paths = jTreeComposite.getSelectionPaths();
            for (int i = paths.length - 1; i >= 0; i--) {
                Item node = (Item) paths[i].getLastPathComponent();
                CompositeItem parent = node.getParent();
                if (parent != null) {
                    parent.removeItem(node.getName());
                    if (parent.getItemsLength() == 0) {
                        CompositeItem grandfather = parent.getParent();
                        if (grandfather != null) {
                            int index = grandfather.getItemIndex(parent.getName());
                            grandfather.removeItem(parent.getName());
                            Item novo = parent.toItem();
                            grandfather.addItem(index, novo);
                        }
                    }
                    jTreeComposite.updateUI();
                    jTreeComposite.expandAll();
                    jTreeComposite.setSelectionPath(paths[i].getParentPath());
                }
            }
        }
    }

    void jTreeComposite_singleClick(JTreeSmartEvent e) {
        Item item = (Item) e.path.getLastPathComponent();
        jPanelItem.setItem(item);
    }

    public CompositeItem getRootItem() {
        return rootItem;
    }

    public void setRootItem(CompositeItem rootItem) {
        this.rootItem = rootItem;
    }

    public Item getSelectedItem() {
        return jPanelItem.getItem();
    }
}