/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare.gui;

import java.util.*;

import javax.swing.event.*;
import javax.swing.tree.*;

import org.simulare.*;

/**
 * The tree model for composite items.
 */
public class CompositeItemTreeModel implements TreeModel {
    private CompositeItem root;
    private transient Vector treeModelListeners;

    public CompositeItemTreeModel(CompositeItem root) {
        this.root = root;
    }

    public Object getRoot() {
        return root;
    }

    public Object getChild(Object parent, int index) {
        if (parent instanceof CompositeItem) {
            CompositeItem p = (CompositeItem) parent;
            return p.getItem(index);
        }
        return null;
    }

    public int getChildCount(Object parent) {
        if (parent instanceof CompositeItem) {
            CompositeItem p = (CompositeItem) parent;
            return p.getItemsLength();
        }
        return 0;
    }

    public boolean isLeaf(Object node) {
        return getChildCount(node) == 0;
    }

    public int getIndexOfChild(Object parent, Object child) {
        if (parent instanceof CompositeItem) {
            CompositeItem p = (CompositeItem) parent;
            return p.getItemIndex(((Item) child).getName());
        }
        return -1;
    }

    public void valueForPathChanged(TreePath path, Object newValue) {
        //System.out.println("PATH:"+path+", VALUE:"+newValue);
        fireTreeNodesChanged(new TreeModelEvent(this, path));
    }

    public synchronized void removeTreeModelListener(TreeModelListener l) {
        if (treeModelListeners != null && treeModelListeners.contains(l)) {
            Vector v = (Vector) treeModelListeners.clone();
            v.removeElement(l);
            treeModelListeners = v;
        }
    }

    public synchronized void addTreeModelListener(TreeModelListener l) {
        Vector v = treeModelListeners == null ? new Vector(2) :
                   (Vector) treeModelListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            treeModelListeners = v;
        }
    }

    protected void fireTreeNodesChanged(TreeModelEvent e) {
        if (treeModelListeners != null) {
            Vector listeners = treeModelListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((TreeModelListener) listeners.elementAt(i)).treeNodesChanged(e);
            }
        }
    }

    protected void fireTreeNodesInserted(TreeModelEvent e) {
        if (treeModelListeners != null) {
            Vector listeners = treeModelListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((TreeModelListener) listeners.elementAt(i)).treeNodesInserted(
                        e);
            }
        }
    }

    protected void fireTreeNodesRemoved(TreeModelEvent e) {
        if (treeModelListeners != null) {
            Vector listeners = treeModelListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((TreeModelListener) listeners.elementAt(i)).treeNodesRemoved(e);
            }
        }
    }

    protected void fireTreeStructureChanged(TreeModelEvent e) {
        if (treeModelListeners != null) {
            Vector listeners = treeModelListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((TreeModelListener) listeners.elementAt(i)).
                        treeStructureChanged(e);
            }
        }
    }
}