/*
 * Simulare is the legal property of USINA (http://usina.tigris.org),
 * and all those who contributed to the development of this tool. Please
 * refer to the COPYRIGHT file distributed with this source distribution
 * for a list of contributors.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or(at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package org.simulare.gui;

import java.awt.*;
import java.util.*;

import javax.swing.*;
import java.awt.event.*;

/**
 * A GUI contorl to add/remove elements and up/down order.
 */
public class Control extends JPanel {
    GridLayout gridLayout1 = new GridLayout();
    JButton jButtonAdd = new JButton();
    JButton jButtonRemove = new JButton();
    JButton jButtonUp = new JButton();
    JButton jButtonDown = new JButton();
    ImageIcon add;
    ImageIcon remove;
    ImageIcon up;
    ImageIcon down;
    private transient Vector controlListeners;

    public Control() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        add = new ImageIcon(org.simulare.gui.Control.class.getResource(
                "New16.gif"));
        remove = new ImageIcon(org.simulare.gui.Control.class.getResource(
                "Remove16.gif"));
        up = new ImageIcon(org.simulare.gui.Control.class.getResource(
                "Up16.gif"));
        down = new ImageIcon(org.simulare.gui.Control.class.getResource(
                "Down16.gif"));
        jButtonAdd.setIcon(add);
        jButtonAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonAdd_actionPerformed(e);
            }
        });
        jButtonRemove.setIcon(remove);
        jButtonRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonRemove_actionPerformed(e);
            }
        });
        jButtonUp.setIcon(up);
        jButtonUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonUp_actionPerformed(e);
            }
        });
        jButtonDown.setIcon(down);
        jButtonDown.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonDown_actionPerformed(e);
            }
        });
        this.setLayout(gridLayout1);
        gridLayout1.setColumns(1);
        gridLayout1.setHgap(2);
        gridLayout1.setRows(4);
        gridLayout1.setVgap(2);
        this.add(jButtonAdd, null);
        this.add(jButtonRemove, null);
        this.add(jButtonUp, null);
        this.add(jButtonDown, null);
    }

    public synchronized void removeControlListener(ControlListener l) {
        if (controlListeners != null && controlListeners.contains(l)) {
            Vector v = (Vector) controlListeners.clone();
            v.removeElement(l);
            controlListeners = v;
        }
    }

    public synchronized void addControlListener(ControlListener l) {
        Vector v = controlListeners == null ? new Vector(2) :
                   (Vector) controlListeners.clone();
        if (!v.contains(l)) {
            v.addElement(l);
            controlListeners = v;
        }
    }

    protected void fireAddClicked(ControlEvent e) {
        if (controlListeners != null) {
            Vector listeners = controlListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((ControlListener) listeners.elementAt(i)).addClicked(e);
            }
        }
    }

    protected void fireRemoveClicked(ControlEvent e) {
        if (controlListeners != null) {
            Vector listeners = controlListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((ControlListener) listeners.elementAt(i)).removeClicked(e);
            }
        }
    }

    protected void fireUpClicked(ControlEvent e) {
        if (controlListeners != null) {
            Vector listeners = controlListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((ControlListener) listeners.elementAt(i)).upClicked(e);
            }
        }
    }

    protected void fireDownClicked(ControlEvent e) {
        if (controlListeners != null) {
            Vector listeners = controlListeners;
            int count = listeners.size();
            for (int i = 0; i < count; i++) {
                ((ControlListener) listeners.elementAt(i)).downClicked(e);
            }
        }
    }

    void jButtonAdd_actionPerformed(ActionEvent e) {
        fireAddClicked(new ControlEvent(jButtonAdd,e));
    }

    void jButtonRemove_actionPerformed(ActionEvent e) {
        fireRemoveClicked(new ControlEvent(jButtonRemove,e));
    }

    void jButtonUp_actionPerformed(ActionEvent e) {
        fireUpClicked(new ControlEvent(jButtonUp,e));
    }

    void jButtonDown_actionPerformed(ActionEvent e) {
        fireDownClicked(new ControlEvent(jButtonDown,e));
    }
}